<?php
/**
 * @copyright	Copyright (C) 2005 - 2013 Open Source Matters, Inc. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access.
defined('_JEXEC') or die;
/**
 * Example Extension Plugin
 *
 * @package		Joomla.Plugin
 * @subpackage	Extension.Example
 * @since		1.5
 */
class plgExtensionUniplaceInstaller extends JPlugin
{
	/**
	 * @var PATH_TO_CODE string   	путь к папке с кодом php-клиента uniplace
	 * @var CONFIG_FILE_NAME string  имя файла конфига // uniplacer_config.php
	 * @var DEF_CHARSET string  		кодировка по умолчанию
	 */
	const PATH_TO_CODE = 'codes/php/uniplace_id/';
	const CONFIG_FILE_NAME = 'uniplacer_config.php';
	const DEF_CHARSET = 'utf-8';

	/**
	 * Allow to processing of extension data after it is saved.
	 *
	 * @param	object	The data representing the extension.
	 * @param	boolean	True is this is new data, false if it is existing data.
	 * @since	1.6
	 */
	function onExtensionAfterSave($context, &$table, $isNew)
	{
		$moduleName = $table->module;
		
		if (!$isNew)
		{
			if ('mod_uniplace_installer' == $moduleName)
			{
				$moduleParams = json_decode($table->params);
				$uniplaceSiteHash = trim($moduleParams->uniplace_site_hash);
				
				// @fixme: плохая практика - в методе create_config_file() сохранения конфига вытащить из настроек модуля mod_uniplace_installer
				$GLOBALS['uniplace_uniplace_charset'] = (trim($moduleParams->uniplace_charset) != '') ? $moduleParams->uniplace_charset : self::DEF_CHARSET;	// @todo: DEF_CHARSET брать из установки модуля
				
				if ($uniplaceSiteHash != '')
				{
					$ret = self::code_files_install($uniplaceSiteHash);
				}
			}
		}
	}

	/**
	 * инсталлирует файлы php-клиента системы uniplace с $uniplace_site_hash в корень сайта
	 * создан для будущего расширения функциональности
	 * 
	 * @param  string $uniplace_site_hash хеш сайта в системе uniplace
	 * @return bool
	 */
	public static function code_files_install($uniplace_site_hash)
	{
		$ret = false;
		$type_mess = '';
		$txt = '';
		
		if ($uniplace_site_hash != '')
		{
			$ret = self::code_files_copy($uniplace_site_hash);
			if ($ret)
			{
				JLog::add(JText::_('PLG_SUCCESS_CODE_FILES_INSTALLED'), JLog::INFO);
			}
			else
			{
				JLog::add(JText::_('PLG_ERROR_CODE_FILES_NOT_INSTALLED'), JLog::WARNING, 'jerror');
			}
		}
		else
		{
			JLog::add(JText::_('PLG_ERROR_UNIPLACE_SITE_HASH_IS_EMPTY'), JLog::ERROR, 'jerror');
		}

		return $ret;
	}
	
	/**
	 * создает папку с именем $uniplace_site_hash в корне сайта с правами 0755
	 * копирует файлы php-клиента системы uniplace в папку с именем $uniplace_site_hash 
	 * 
	 * @param  string $uniplace_site_hash хеш сайта в системе uniplace
	 * @return bool
	 */
	public static function code_files_copy($uniplace_site_hash)
	{
		$uniplace_site_hash = trim($uniplace_site_hash);
		$ret = false;
		
		if ($uniplace_site_hash != '')
		{
			$ret = self::create_config_file($uniplace_site_hash);
			
			if ($ret)
			{
				$path = self::get_document_root() . '/' . $uniplace_site_hash;
				$path = self::prepare_path($path);	// windows
				
				if (!file_exists($path))
				{
					$ret = mkdir($path, 0755);
					
					if (true === $ret)
					{
						$ret = chmod($path, 0755);
						
						if (!$ret)
						{
							// $err = sprintf('Warning: Can\'t change folder permissions to 0755 and work may be incorrect. Please change folder permissions to 0755 %s', $path);
							JLog::add(JText::sprintf('PLG_WARNING_CANT_CHANGE_FOLDER_PERMISSIONS', $path), JLog::WARNING, 'jerror');
						}
						
						$path_to_code = (__DIR__) . '/' . self::PATH_TO_CODE;
						$path_to_code = self::prepare_path($path_to_code);	// windows
					
						$files = scandir($path_to_code);
						foreach ($files as $file)
						{
							if ($file != "." && $file != "..")							{
								copy("$path_to_code/$file", "$path/$file"); 
							}
						}
					}
					else
					{
						// $err = 'Error: Can\'t create directory with Uniplace client. Please check parent folder permissions';
						JLog::add(JText::_('PLG_ERROR_CANT_CREATE_DIRECTORY'), JLog::WARNING, 'jerror');
					}
				}
				else
				{
					// $err = sprintf('Error: Directory with Uniplace client exists already. Please delete this folder %s', $path);
					JLog::add(JText::sprintf('PLG_WARNING_DIRECTORY_EXISTS_ALREADY', $path), JLog::WARNING, 'jerror');
				}
			}
			else
			{
				// $err = sprintf('Error: Can\'t create config file - %s Please check permissions for folder or this file if this file exists already.', $_SERVER['DOCUMENT_ROOT'] . '/' . self::CONFIG_FILE_NAME);
				JLog::add(JText::sprintf('PLG_ERROR_CANT_CREATE_CONFIG_FILE', self::get_document_root() . '/' . self::CONFIG_FILE_NAME), JLog::WARNING, 'jerror');
			}
		}
		else
		{
			// $err = 'Error: Uniplace site hash is empty. Please enter the Uniplace site hash.';
			JLog::add(JText::_('PLG_ERROR_UNIPLACE_SITE_HASH_IS_EMPTY'), JLog::ERROR, 'jerror');
		}
		
		return $ret;
	}
	
	/**
	 * заменяет в строке пути $path обратные слеши на прямые
	 * для окружения на платфоме windows
	 * 
	 * @return string
	 */
	public static function prepare_path($path)
	{
		return str_replace('\\', '/', $path);	// windows
	}
	
	/**
	 * формирует и создает файл конфигурации self::CONFIG_FILE_NAME в корне сайта
	 * 
	 * @param  string $uniplace_site_hash хеш сайта в системе uniplace
	 * @return bool
	 */
	public static function create_config_file($uniplace_site_hash)
	{
		$uniplace_site_hash = trim($uniplace_site_hash);
		
		// @fixme: вытащить из настроек модуля mod_uniplace_installer
		// $params = JModuleHelper::getParams('mod_uniplace_installer');  //self::DEF_CHARSET
		// $uniplace_charset = $params->get('uniplace_charset');
		
		$uniplace_charset = $GLOBALS['uniplace_uniplace_charset'];
		
		$ret = false;
		
		if ($uniplace_site_hash != '')
		{
			$content_config_file = "<?php" . PHP_EOL;
			$content_config_file .= "\tdefine('_UNIPLACE_USER_', '$uniplace_site_hash'); // Site hash" . PHP_EOL;
			$content_config_file .= "\tdefine('_UNIPLACE_CHARSET_', '$uniplace_charset'); // Anchor text encoding" . PHP_EOL;
			$content_config_file .= "?>";
			
			$config_file = self::get_document_root() . '/' . self::CONFIG_FILE_NAME;
			
			$ret = file_put_contents($config_file, $content_config_file);
			
			if ($ret)
			{
				$res_permissions = chmod($config_file, 0755);
				if (!$res_permissions)
				{
					// $err = sprintf('Warning: Can\'t change config file permissions to 0755 and work may be incorrect. Please change config file permissions to 0755 %s', $config_file);
					JLog::add(JText::sprintf('PLG_WARNING_CANT_CHANGE_CONFIG_FILE_PERMISSIONS', $config_file), JLog::WARNING, 'jerror');
				}
			}
		}
		else
		{
			// $err = 'Error: Uniplace site hash is empty. Please enter the Uniplace site hash.';
			JLog::add(JText::_('PLG_ERROR_UNIPLACE_SITE_HASH_IS_EMPTY'), JLog::ERROR, 'jerror');
		}
		
		return ((false === $ret) ? false : true);
	}

	/**
	 * возвращает корректный DOCUMENT_ROOT
	 * в некоторых системах может быть ненастроен
	 * 
	 * @return bool|string
	 */
	public static function get_document_root()
	{
		if (defined('JPATH_ROOT'))
		{
			return JPATH_ROOT;
		}
		else
		{
			return trim(dirname(realpath(getcwd())));
		}
	}
}
